import sys
import typing


def primitive_nurbs_surface_circle_add(
        radius: float = 1.0,
        enter_editmode: bool = False,
        align: typing.Union[str, int] = 'WORLD',
        location: typing.List[float] = (0.0, 0.0, 0.0),
        rotation: typing.List[float] = (0.0, 0.0, 0.0)):
    ''' Construct a Nurbs surface Circle

    :param radius: Radius
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object
    :type enter_editmode: bool
    :param align: Align, The alignment of the new object * WORLD World, Align the new object to the world. * VIEW View, Align the new object to the view. * CURSOR 3D Cursor, Use the 3D cursor orientation for the new object.
    :type align: typing.Union[str, int]
    :param location: Location, Location for the newly added object
    :type location: typing.List[float]
    :param rotation: Rotation, Rotation for the newly added object
    :type rotation: typing.List[float]
    '''

    pass


def primitive_nurbs_surface_curve_add(
        radius: float = 1.0,
        enter_editmode: bool = False,
        align: typing.Union[str, int] = 'WORLD',
        location: typing.List[float] = (0.0, 0.0, 0.0),
        rotation: typing.List[float] = (0.0, 0.0, 0.0)):
    ''' Construct a Nurbs surface Curve

    :param radius: Radius
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object
    :type enter_editmode: bool
    :param align: Align, The alignment of the new object * WORLD World, Align the new object to the world. * VIEW View, Align the new object to the view. * CURSOR 3D Cursor, Use the 3D cursor orientation for the new object.
    :type align: typing.Union[str, int]
    :param location: Location, Location for the newly added object
    :type location: typing.List[float]
    :param rotation: Rotation, Rotation for the newly added object
    :type rotation: typing.List[float]
    '''

    pass


def primitive_nurbs_surface_cylinder_add(
        radius: float = 1.0,
        enter_editmode: bool = False,
        align: typing.Union[str, int] = 'WORLD',
        location: typing.List[float] = (0.0, 0.0, 0.0),
        rotation: typing.List[float] = (0.0, 0.0, 0.0)):
    ''' Construct a Nurbs surface Cylinder

    :param radius: Radius
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object
    :type enter_editmode: bool
    :param align: Align, The alignment of the new object * WORLD World, Align the new object to the world. * VIEW View, Align the new object to the view. * CURSOR 3D Cursor, Use the 3D cursor orientation for the new object.
    :type align: typing.Union[str, int]
    :param location: Location, Location for the newly added object
    :type location: typing.List[float]
    :param rotation: Rotation, Rotation for the newly added object
    :type rotation: typing.List[float]
    '''

    pass


def primitive_nurbs_surface_sphere_add(
        radius: float = 1.0,
        enter_editmode: bool = False,
        align: typing.Union[str, int] = 'WORLD',
        location: typing.List[float] = (0.0, 0.0, 0.0),
        rotation: typing.List[float] = (0.0, 0.0, 0.0)):
    ''' Construct a Nurbs surface Sphere

    :param radius: Radius
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object
    :type enter_editmode: bool
    :param align: Align, The alignment of the new object * WORLD World, Align the new object to the world. * VIEW View, Align the new object to the view. * CURSOR 3D Cursor, Use the 3D cursor orientation for the new object.
    :type align: typing.Union[str, int]
    :param location: Location, Location for the newly added object
    :type location: typing.List[float]
    :param rotation: Rotation, Rotation for the newly added object
    :type rotation: typing.List[float]
    '''

    pass


def primitive_nurbs_surface_surface_add(
        radius: float = 1.0,
        enter_editmode: bool = False,
        align: typing.Union[str, int] = 'WORLD',
        location: typing.List[float] = (0.0, 0.0, 0.0),
        rotation: typing.List[float] = (0.0, 0.0, 0.0)):
    ''' Construct a Nurbs surface Patch

    :param radius: Radius
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object
    :type enter_editmode: bool
    :param align: Align, The alignment of the new object * WORLD World, Align the new object to the world. * VIEW View, Align the new object to the view. * CURSOR 3D Cursor, Use the 3D cursor orientation for the new object.
    :type align: typing.Union[str, int]
    :param location: Location, Location for the newly added object
    :type location: typing.List[float]
    :param rotation: Rotation, Rotation for the newly added object
    :type rotation: typing.List[float]
    '''

    pass


def primitive_nurbs_surface_torus_add(
        radius: float = 1.0,
        enter_editmode: bool = False,
        align: typing.Union[str, int] = 'WORLD',
        location: typing.List[float] = (0.0, 0.0, 0.0),
        rotation: typing.List[float] = (0.0, 0.0, 0.0)):
    ''' Construct a Nurbs surface Torus

    :param radius: Radius
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object
    :type enter_editmode: bool
    :param align: Align, The alignment of the new object * WORLD World, Align the new object to the world. * VIEW View, Align the new object to the view. * CURSOR 3D Cursor, Use the 3D cursor orientation for the new object.
    :type align: typing.Union[str, int]
    :param location: Location, Location for the newly added object
    :type location: typing.List[float]
    :param rotation: Rotation, Rotation for the newly added object
    :type rotation: typing.List[float]
    '''

    pass



import sys
import typing


def assign_default_button():
    ''' Set this property's current value as the new default

    '''

    pass


def button_execute(skip_depressed: bool = False):
    ''' Presses active button

    :param skip_depressed: Skip Depressed
    :type skip_depressed: bool
    '''

    pass


def copy_data_path_button(full_path: bool = False):
    ''' Copy the RNA data path for this property to the clipboard

    :param full_path: full_path, Copy full data path
    :type full_path: bool
    '''

    pass


def copy_python_command_button():
    ''' Copy the Python command matching this button

    '''

    pass


def copy_to_selected_button(all: bool = True):
    ''' Copy property from this object to selected objects or bones

    :param all: All, Copy to selected all elements of the array
    :type all: bool
    '''

    pass


def drop_color(color: typing.List[float] = (0.0, 0.0, 0.0),
               gamma: bool = False):
    ''' Drop colors to buttons

    :param color: Color, Source color
    :type color: typing.List[float]
    :param gamma: Gamma Corrected, The source color is gamma corrected
    :type gamma: bool
    '''

    pass


def editsource():
    ''' Edit UI source code of the active button

    '''

    pass


def edittranslation_init():
    ''' Edit i18n in current language for the active button

    '''

    pass


def eyedropper_color(use_accumulate: bool = True):
    ''' Sample a color from the Blender Window to store in a property

    :param use_accumulate: Accumulate
    :type use_accumulate: bool
    '''

    pass


def eyedropper_colorramp():
    ''' Sample a color band

    '''

    pass


def eyedropper_colorramp_point():
    ''' Point-sample a color band

    '''

    pass


def eyedropper_depth():
    ''' Sample depth from the 3D view

    '''

    pass


def eyedropper_driver(mapping_type: typing.Union[str, int] = 'SINGLE_MANY'):
    ''' Pick a property to use as a driver target

    :param mapping_type: Mapping Type, Method used to match target and driven properties * SINGLE_MANY All from Target, Drive all components of this property using the target picked. * DIRECT Single from Target, Drive this component of this property using the target picked. * MATCH Match Indices, Create drivers for each pair of corresponding elements. * NONE_ALL Manually Create Later, Create drivers for all properties without assigning any targets yet. * NONE_SINGLE Manually Create Later (Single), Create driver for this property only and without assigning any targets yet.
    :type mapping_type: typing.Union[str, int]
    '''

    pass


def eyedropper_id():
    ''' Sample a data-block from the 3D View to store in a property

    '''

    pass


def jump_to_target_button():
    ''' Switch to the target object or bone

    '''

    pass


def override_remove_button(all: bool = True):
    ''' Remove an override operation

    :param all: All, Reset to default values all elements of the array
    :type all: bool
    '''

    pass


def override_type_set_button(all: bool = True,
                             type: typing.Union[str, int] = 'REPLACE'):
    ''' Create an override operation, or set the type of an existing one

    :param all: All, Reset to default values all elements of the array
    :type all: bool
    :param type: Type, Type of override operation * NOOP NoOp, 'No-Operation', place holder preventing automatic override to ever affect the property. * REPLACE Replace, Completely replace value from linked data by local one. * DIFFERENCE Difference, Store difference to linked data value. * FACTOR Factor, Store factor to linked data value (useful e.g. for scale).
    :type type: typing.Union[str, int]
    '''

    pass


def reloadtranslation():
    ''' Force a full reload of UI translation

    '''

    pass


def reports_to_textblock():
    ''' Write the reports

    '''

    pass


def reset_default_button(all: bool = True):
    ''' Reset this property's value to its default value

    :param all: All, Reset to default values all elements of the array
    :type all: bool
    '''

    pass


def unset_property_button():
    ''' Clear the property and use default or generated value in operators

    '''

    pass



import sys
import typing


class Device:
    ''' Device objects represent an audio output backend like OpenAL or SDL, but might also represent a file output or RAM buffer output. lock() Locks the device so that it's guaranteed, that no samples are read from the streams until :meth: unlock is called. This is useful if you want to do start/stop/pause/resume some sounds at the same time. play(sound, keep=False) Plays a sound. :arg sound: The sound to play. :type sound: Sound :arg keep: See :attr: Handle.keep . :type keep: bool :return: The playback handle with which playback can be controlled with. :rtype: Handle stopAll() Stops all playing and paused sounds. unlock() Unlocks the device after a lock call, see :meth: lock for details.
    '''

    channels = None
    ''' The channel count of the device.'''

    distance_model = None
    ''' The distance model of the device.'''

    doppler_factor = None
    ''' The doppler factor of the device. This factor is a scaling factor for the velocity vectors in doppler calculation. So a value bigger than 1 will exaggerate the effect as it raises the velocity.'''

    format = None
    ''' The native sample format of the device.'''

    listener_location = None
    ''' The listeners's location in 3D space, a 3D tuple of floats.'''

    listener_orientation = None
    ''' The listener's orientation in 3D space as quaternion, a 4 float tuple.'''

    listener_velocity = None
    ''' The listener's velocity in 3D space, a 3D tuple of floats.'''

    rate = None
    ''' The sampling rate of the device in Hz.'''

    speed_of_sound = None
    ''' The speed of sound of the device. The speed of sound in air is typically 343.3 m/s.'''

    volume = None
    ''' The overall volume of the device.'''


class DynamicMusic:
    ''' The DynamicMusic object allows to play music depending on a current scene, scene changes are managed by the class, with the possibility of custom transitions. The default transition is a crossfade effect, and the default scene is silent and has id 0 addScene(scene) Adds a new scene. :arg scene: The scene sound. :type scene: Sound :return: The new scene id. :rtype: int addTransition(ini, end, transition) Adds a new scene. :arg ini: the initial scene foor the transition. :type ini: int :arg end: The final scene for the transition. :type end: int :arg transition: The transition sound. :type transition: Sound :return: false if the ini or end scenes don't exist, true othrwise. :rtype: bool pause() Pauses playback of the scene. :return: Whether the action succeeded. :rtype: bool resume() Resumes playback of the scene. :return: Whether the action succeeded. :rtype: bool stop() Stops playback of the scene. :return: Whether the action succeeded. :rtype: bool
    '''

    fadeTime = None
    ''' The length in seconds of the crossfade transition'''

    position = None
    ''' The playback position of the scene in seconds.'''

    scene = None
    ''' The current scene'''

    status = None
    ''' Whether the scene is playing, paused or stopped (=invalid).'''

    volume = None
    ''' The volume of the scene.'''


class Handle:
    ''' Handle objects are playback handles that can be used to control playback of a sound. If a sound is played back multiple times then there are as many handles. pause() Pauses playback. :return: Whether the action succeeded. :rtype: bool resume() Resumes playback. :return: Whether the action succeeded. :rtype: bool stop() Stops playback. :return: Whether the action succeeded. :rtype: bool
    '''

    attenuation = None
    ''' This factor is used for distance based attenuation of the source.'''

    cone_angle_inner = None
    ''' The opening angle of the inner cone of the source. If the cone values of a source are set there are two (audible) cones with the apex at the :attr: location of the source and with infinite height, heading in the direction of the source's :attr: orientation . In the inner cone the volume is normal. Outside the outer cone the volume will be :attr: cone_volume_outer and in the area between the volume will be interpolated linearly.'''

    cone_angle_outer = None
    ''' The opening angle of the outer cone of the source.'''

    cone_volume_outer = None
    ''' The volume outside the outer cone of the source.'''

    distance_maximum = None
    ''' The maximum distance of the source. If the listener is further away the source volume will be 0.'''

    distance_reference = None
    ''' The reference distance of the source. At this distance the volume will be exactly :attr: volume .'''

    keep = None
    ''' Whether the sound should be kept paused in the device when its end is reached. This can be used to seek the sound to some position and start playback again.'''

    location = None
    ''' The source's location in 3D space, a 3D tuple of floats.'''

    loop_count = None
    ''' The (remaining) loop count of the sound. A negative value indicates infinity.'''

    orientation = None
    ''' The source's orientation in 3D space as quaternion, a 4 float tuple.'''

    pitch = None
    ''' The pitch of the sound.'''

    position = None
    ''' The playback position of the sound in seconds.'''

    relative = None
    ''' Whether the source's location, velocity and orientation is relative or absolute to the listener.'''

    status = None
    ''' Whether the sound is playing, paused or stopped (=invalid).'''

    velocity = None
    ''' The source's velocity in 3D space, a 3D tuple of floats.'''

    volume = None
    ''' The volume of the sound.'''

    volume_maximum = None
    ''' The maximum volume of the source.'''

    volume_minimum = None
    ''' The minimum volume of the source.'''


class PlaybackManager:
    ''' A PlabackManager object allows to easily control groups os sounds organized in categories. addCategory(volume) Adds a category with a custom volume. :arg volume: The volume for ther new category. :type volume: float :return: The key of the new category. :rtype: int clean() Cleans all the invalid and finished sound from the playback manager. getVolume(catKey) Retrieves the volume of a category. :arg catKey: the key of the category. :type catKey: int :return: The volume of the cateogry. :rtype: float pause(catKey) Pauses playback of the category. :arg catKey: the key of the category. :type catKey: int :return: Whether the action succeeded. :rtype: bool setVolume(sound, catKey) Plays a sound through the playback manager and assigns it to a category. :arg sound: The sound to play. :type sound: Sound :arg catKey: the key of the category in which the sound will be added, if it doesn't exist, a new one will be created. :type catKey: int :return: The playback handle with which playback can be controlled with. :rtype: Handle resume(catKey) Resumes playback of the catgory. :arg catKey: the key of the category. :type catKey: int :return: Whether the action succeeded. :rtype: bool setVolume(volume, catKey) Changes the volume of a category. :arg volume: the new volume value. :type volume: float :arg catKey: the key of the category. :type catKey: int :return: Whether the action succeeded. :rtype: int stop(catKey) Stops playback of the category. :arg catKey: the key of the category. :type catKey: int :return: Whether the action succeeded. :rtype: bool
    '''

    pass


class Sequence:
    ''' This sound represents sequenced entries to play a sound sequence. add() Adds a new entry to the sequence. :arg sound: The sound this entry should play. :type sound: Sound :arg begin: The start time. :type begin: float :arg end: The end time or a negative value if determined by the sound. :type end: float :arg skip: How much seconds should be skipped at the beginning. :type skip: float :return: The entry added. :rtype: SequenceEntry remove() Removes an entry from the sequence. :arg entry: The entry to remove. :type entry: SequenceEntry setAnimationData() Writes animation data to a sequence. :arg type: The type of animation data. :type type: int :arg frame: The frame this data is for. :type frame: int :arg data: The data to write. :type data: sequence of float :arg animated: Whether the attribute is animated. :type animated: bool
    '''

    channels = None
    ''' The channel count of the sequence.'''

    distance_model = None
    ''' The distance model of the sequence.'''

    doppler_factor = None
    ''' The doppler factor of the sequence. This factor is a scaling factor for the velocity vectors in doppler calculation. So a value bigger than 1 will exaggerate the effect as it raises the velocity.'''

    fps = None
    ''' The listeners's location in 3D space, a 3D tuple of floats.'''

    muted = None
    ''' Whether the whole sequence is muted.'''

    rate = None
    ''' The sampling rate of the sequence in Hz.'''

    speed_of_sound = None
    ''' The speed of sound of the sequence. The speed of sound in air is typically 343.3 m/s.'''


class SequenceEntry:
    ''' SequenceEntry objects represent an entry of a sequenced sound. move() Moves the entry. :arg begin: The new start time. :type begin: float :arg end: The new end time or a negative value if unknown. :type end: float :arg skip: How many seconds to skip at the beginning. :type skip: float setAnimationData() Writes animation data to a sequenced entry. :arg type: The type of animation data. :type type: int :arg frame: The frame this data is for. :type frame: int :arg data: The data to write. :type data: sequence of float :arg animated: Whether the attribute is animated. :type animated: bool
    '''

    attenuation = None
    ''' This factor is used for distance based attenuation of the source.'''

    cone_angle_inner = None
    ''' The opening angle of the inner cone of the source. If the cone values of a source are set there are two (audible) cones with the apex at the :attr: location of the source and with infinite height, heading in the direction of the source's :attr: orientation . In the inner cone the volume is normal. Outside the outer cone the volume will be :attr: cone_volume_outer and in the area between the volume will be interpolated linearly.'''

    cone_angle_outer = None
    ''' The opening angle of the outer cone of the source.'''

    cone_volume_outer = None
    ''' The volume outside the outer cone of the source.'''

    distance_maximum = None
    ''' The maximum distance of the source. If the listener is further away the source volume will be 0.'''

    distance_reference = None
    ''' The reference distance of the source. At this distance the volume will be exactly :attr: volume .'''

    muted = None
    ''' Whether the entry is muted.'''

    relative = None
    ''' Whether the source's location, velocity and orientation is relative or absolute to the listener.'''

    sound = None
    ''' The sound the entry is representing and will be played in the sequence.'''

    volume_maximum = None
    ''' The maximum volume of the source.'''

    volume_minimum = None
    ''' The minimum volume of the source.'''


class Sound:
    ''' Sound objects are immutable and represent a sound that can be played simultaneously multiple times. They are called factories because they create reader objects internally that are used for playback. buffer(data, rate) Creates a sound from a data buffer. :arg data: The data as two dimensional numpy array. :type data: numpy.ndarray :arg rate: The sample rate. :type rate: double :return: The created Sound object. :rtype: Sound file(filename) Creates a sound object of a sound file. :arg filename: Path of the file. :type filename: string :return: The created Sound object. :rtype: Sound list() Creates an empty sound list that can contain several sounds. :arg random: wether the playback will be random or not. :type random: int :return: The created Sound object. :rtype: Sound sawtooth(frequency, rate=48000) Creates a sawtooth sound which plays a sawtooth wave. :arg frequency: The frequency of the sawtooth wave in Hz. :type frequency: float :arg rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping. :type rate: int :return: The created Sound object. :rtype: Sound silence(rate=48000) Creates a silence sound which plays simple silence. :arg rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping. :type rate: int :return: The created Sound object. :rtype: Sound sine(frequency, rate=48000) Creates a sine sound which plays a sine wave. :arg frequency: The frequency of the sine wave in Hz. :type frequency: float :arg rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping. :type rate: int :return: The created Sound object. :rtype: Sound square(frequency, rate=48000) Creates a square sound which plays a square wave. :arg frequency: The frequency of the square wave in Hz. :type frequency: float :arg rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping. :type rate: int :return: The created Sound object. :rtype: Sound triangle(frequency, rate=48000) Creates a triangle sound which plays a triangle wave. :arg frequency: The frequency of the triangle wave in Hz. :type frequency: float :arg rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping. :type rate: int :return: The created Sound object. :rtype: Sound ADSR(attack,decay,sustain,release) Attack-Decay-Sustain-Release envelopes the volume of a sound. Note: there is currently no way to trigger the release with this API. :arg attack: The attack time in seconds. :type attack: float :arg decay: The decay time in seconds. :type decay: float :arg sustain: The sustain level. :type sustain: float :arg release: The release level. :type release: float :return: The created Sound object. :rtype: Sound accumulate(additive=False) Accumulates a sound by summing over positive input differences thus generating a monotonic sigal. If additivity is set to true negative input differences get added too, but positive ones with a factor of two. Note that with additivity the signal is not monotonic anymore. :arg additive: Whether the accumulation should be additive or not. :type time: bool :return: The created Sound object. :rtype: Sound addSound(sound) Adds a new sound to a sound list. :arg sound: The sound that will be added to the list. :type sound: Sound cache() Caches a sound into RAM. This saves CPU usage needed for decoding and file access if the underlying sound reads from a file on the harddisk, but it consumes a lot of memory. :return: The created Sound object. :rtype: Sound data() Retrieves the data of the sound as numpy array. :return: A two dimensional numpy float array. :rtype: numpy.ndarray delay(time) Delays by playing adding silence in front of the other sound's data. :arg time: How many seconds of silence should be added before the sound. :type time: float :return: The created Sound object. :rtype: Sound envelope(attack, release, threshold, arthreshold) Delays by playing adding silence in front of the other sound's data. :arg attack: The attack factor. :type attack: float :arg release: The release factor. :type release: float :arg threshold: The general threshold value. :type threshold: float :arg arthreshold: The attack/release threshold value. :type arthreshold: float :return: The created Sound object. :rtype: Sound fadein(start, length) Fades a sound in by raising the volume linearly in the given time interval. :arg start: Time in seconds when the fading should start. :type start: float :arg length: Time in seconds how long the fading should last. :type length: float :return: The created Sound object. :rtype: Sound fadeout(start, length) Fades a sound in by lowering the volume linearly in the given time interval. :arg start: Time in seconds when the fading should start. :type start: float :arg length: Time in seconds how long the fading should last. :type length: float :return: The created Sound object. :rtype: Sound filter(b, a = (1)) Filters a sound with the supplied IIR filter coefficients. Without the second parameter you'll get a FIR filter. If the first value of the a sequence is 0 it will be set to 1 automatically. If the first value of the a sequence is neither 0 nor 1, all filter coefficients will be scaled by this value so that it is 1 in the end, you don't have to scale yourself. :arg b: The nominator filter coefficients. :type b: sequence of float :arg a: The denominator filter coefficients. :type a: sequence of float :return: The created Sound object. :rtype: Sound highpass(frequency, Q=0.5) Creates a second order highpass filter based on the transfer function H(s) = s^2 / (s^2 + s/Q + 1) :arg frequency: The cut off trequency of the highpass. :type frequency: float :arg Q: Q factor of the lowpass. :type Q: float :return: The created Sound object. :rtype: Sound join(sound) Plays two factories in sequence. :arg sound: The sound to play second. :type sound: Sound :return: The created Sound object. :rtype: Sound limit(start, end) Limits a sound within a specific start and end time. :arg start: Start time in seconds. :type start: float :arg end: End time in seconds. :type end: float :return: The created Sound object. :rtype: Sound loop(count) Loops a sound. :arg count: How often the sound should be looped. Negative values mean endlessly. :type count: integer :return: The created Sound object. :rtype: Sound lowpass(frequency, Q=0.5) Creates a second order lowpass filter based on the transfer function H(s) = 1 / (s^2 + s/Q + 1) :arg frequency: The cut off trequency of the lowpass. :type frequency: float :arg Q: Q factor of the lowpass. :type Q: float :return: The created Sound object. :rtype: Sound mix(sound) Mixes two factories. :arg sound: The sound to mix over the other. :type sound: Sound :return: The created Sound object. :rtype: Sound modulate(sound) Modulates two factories. :arg sound: The sound to modulate over the other. :type sound: Sound :return: The created Sound object. :rtype: Sound mutable() Creates a sound that will be restarted when sought backwards. If the original sound is a sound list, the playing sound can change. :return: The created Sound object. :rtype: Sound pingpong() Plays a sound forward and then backward. This is like joining a sound with its reverse. :return: The created Sound object. :rtype: Sound pitch(factor) Changes the pitch of a sound with a specific factor. :arg factor: The factor to change the pitch with. :type factor: float :return: The created Sound object. :rtype: Sound rechannel(channels) Rechannels the sound. :arg channels: The new channel configuration. :type channels: int :return: The created Sound object. :rtype: Sound resample(rate, high_quality) Resamples the sound. :arg rate: The new sample rate. :type rate: double :arg high_quality: When true use a higher quality but slower resampler. :type high_quality: bool :return: The created Sound object. :rtype: Sound reverse() Plays a sound reversed. :return: The created Sound object. :rtype: Sound sum() Sums the samples of a sound. :return: The created Sound object. :rtype: Sound threshold(threshold = 0) Makes a threshold wave out of an audio wave by setting all samples with a amplitude >= threshold to 1, all <= -threshold to -1 and all between to 0. :arg threshold: Threshold value over which an amplitude counts non-zero. :type threshold: float :return: The created Sound object. :rtype: Sound volume(volume) Changes the volume of a sound. :arg volume: The new volume.. :type volume: float :return: The created Sound object. :rtype: Sound write(filename, rate, channels, format, container, codec, bitrate, buffersize) Writes the sound to a file. :arg filename: The path to write to. :type filename: string :arg rate: The sample rate to write with. :type rate: int :arg channels: The number of channels to write with. :type channels: int :arg format: The sample format to write with. :type format: int :arg container: The container format for the file. :type container: int :arg codec: The codec to use in the file. :type codec: int :arg bitrate: The bitrate to write with. :type bitrate: int :arg buffersize: The size of the writing buffer. :type buffersize: int
    '''

    length = None
    ''' The sample specification of the sound as a tuple with rate and channel count.'''

    specs = None
    ''' The sample specification of the sound as a tuple with rate and channel count.'''


class Source:
    ''' The source object represents the source position of a binaural sound.
    '''

    azimuth = None
    ''' The azimuth angle.'''

    distance = None
    ''' The distance value. 0 is min, 1 is max.'''

    elevation = None
    ''' The elevation angle.'''


class ThreadPool:
    ''' A ThreadPool is used to parallelize convolution efficiently.
    '''

    pass


class error:
    pass


AP_LOCATION = None
''' constant value 3
'''

AP_ORIENTATION = None
''' constant value 4
'''

AP_PANNING = None
''' constant value 1
'''

AP_PITCH = None
''' constant value 2
'''

AP_VOLUME = None
''' constant value 0
'''

CHANNELS_INVALID = None
''' constant value 0
'''

CHANNELS_MONO = None
''' constant value 1
'''

CHANNELS_STEREO = None
''' constant value 2
'''

CHANNELS_STEREO_LFE = None
''' constant value 3
'''

CHANNELS_SURROUND4 = None
''' constant value 4
'''

CHANNELS_SURROUND5 = None
''' constant value 5
'''

CHANNELS_SURROUND51 = None
''' constant value 6
'''

CHANNELS_SURROUND61 = None
''' constant value 7
'''

CHANNELS_SURROUND71 = None
''' constant value 8
'''

CODEC_AAC = None
''' constant value 1
'''

CODEC_AC3 = None
''' constant value 2
'''

CODEC_FLAC = None
''' constant value 3
'''

CODEC_INVALID = None
''' constant value 0
'''

CODEC_MP2 = None
''' constant value 4
'''

CODEC_MP3 = None
''' constant value 5
'''

CODEC_OPUS = None
''' constant value 8
'''

CODEC_PCM = None
''' constant value 6
'''

CODEC_VORBIS = None
''' constant value 7
'''

CONTAINER_AC3 = None
''' constant value 1
'''

CONTAINER_FLAC = None
''' constant value 2
'''

CONTAINER_INVALID = None
''' constant value 0
'''

CONTAINER_MATROSKA = None
''' constant value 3
'''

CONTAINER_MP2 = None
''' constant value 4
'''

CONTAINER_MP3 = None
''' constant value 5
'''

CONTAINER_OGG = None
''' constant value 6
'''

CONTAINER_WAV = None
''' constant value 7
'''

DISTANCE_MODEL_EXPONENT = None
''' constant value 5
'''

DISTANCE_MODEL_EXPONENT_CLAMPED = None
''' constant value 6
'''

DISTANCE_MODEL_INVALID = None
''' constant value 0
'''

DISTANCE_MODEL_INVERSE = None
''' constant value 1
'''

DISTANCE_MODEL_INVERSE_CLAMPED = None
''' constant value 2
'''

DISTANCE_MODEL_LINEAR = None
''' constant value 3
'''

DISTANCE_MODEL_LINEAR_CLAMPED = None
''' constant value 4
'''

FORMAT_FLOAT32 = None
''' constant value 36
'''

FORMAT_FLOAT64 = None
''' constant value 40
'''

FORMAT_INVALID = None
''' constant value 0
'''

FORMAT_S16 = None
''' constant value 18
'''

FORMAT_S24 = None
''' constant value 19
'''

FORMAT_S32 = None
''' constant value 20
'''

FORMAT_U8 = None
''' constant value 1
'''

RATE_11025 = None
''' constant value 11025
'''

RATE_16000 = None
''' constant value 16000
'''

RATE_192000 = None
''' constant value 192000
'''

RATE_22050 = None
''' constant value 22050
'''

RATE_32000 = None
''' constant value 32000
'''

RATE_44100 = None
''' constant value 44100
'''

RATE_48000 = None
''' constant value 48000
'''

RATE_8000 = None
''' constant value 8000
'''

RATE_88200 = None
''' constant value 88200
'''

RATE_96000 = None
''' constant value 96000
'''

RATE_INVALID = None
''' constant value 0
'''

STATUS_INVALID = None
''' constant value 0
'''

STATUS_PAUSED = None
''' constant value 2
'''

STATUS_PLAYING = None
''' constant value 1
'''

STATUS_STOPPED = None
''' constant value 3
'''



import sys
import typing
import bpy.types


def brush_stroke(stroke: typing.Union[
        typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.
        List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection'] = None,
                 mode: typing.Union[str, int] = 'NORMAL',
                 ignore_background_click: bool = False):
    ''' Sculpt a stroke into the geometry

    :param stroke: Stroke
    :type stroke: typing.Union[typing.Dict[str, 'bpy.types.OperatorStrokeElement'], typing.List['bpy.types.OperatorStrokeElement'], 'bpy_prop_collection']
    :param mode: Stroke Mode, Action taken when a paint stroke is made * NORMAL Regular, Apply brush normally. * INVERT Invert, Invert action of brush for duration of stroke. * SMOOTH Smooth, Switch brush to smooth mode for duration of stroke.
    :type mode: typing.Union[str, int]
    :param ignore_background_click: Ignore Background Click, Clicks on the background do not start the stroke
    :type ignore_background_click: bool
    '''

    pass


def detail_flood_fill():
    ''' Flood fill the mesh with the selected detail setting

    '''

    pass


def dynamic_topology_toggle():
    ''' Dynamic topology alters the mesh topology while sculpting

    '''

    pass


def optimize():
    ''' Recalculate the sculpt BVH to improve performance

    '''

    pass


def sample_detail_size(location: typing.List[int] = (0, 0)):
    ''' Sample the mesh detail on clicked point

    :param location: Location, Screen Coordinates of sampling
    :type location: typing.List[int]
    '''

    pass


def sculptmode_toggle():
    ''' Toggle sculpt mode in 3D view

    '''

    pass


def set_detail_size():
    ''' Set the mesh detail (either relative or constant one, depending on current dyntopo mode)

    '''

    pass


def set_persistent_base():
    ''' Reset the copy of the mesh that is being sculpted on

    '''

    pass


def symmetrize():
    ''' Symmetrize the topology modifications

    '''

    pass


def uv_sculpt_stroke(mode: typing.Union[str, int] = 'NORMAL'):
    ''' Sculpt UVs using a brush

    :param mode: Mode, Stroke Mode * NORMAL Regular, Apply brush normally. * INVERT Invert, Invert action of brush for duration of stroke. * RELAX Relax, Switch brush to relax mode for duration of stroke.
    :type mode: typing.Union[str, int]
    '''

    pass



import sys
import typing
import bpy_types


class SmartProject(bpy_types.Operator):
    bl_idname = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_rna = None
    ''' '''

    id_data = None
    ''' '''

    def as_keywords(self, ignore):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def execute(self, context):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def invoke(self, context, _event):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def poll(self, context):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class thickface:
    pass


def VectoQuat(vec):
    ''' 

    '''

    pass


def boundsIsland(faces):
    ''' 

    '''

    pass


def getUvIslands(faceGroups, me):
    ''' 

    '''

    pass


def island2Edge(island):
    ''' 

    '''

    pass


def islandIntersectUvIsland(source, target, SourceOffset):
    ''' 

    '''

    pass


def main(context, island_margin, projection_limit, user_area_weight,
         use_aspect, stretch_to_bounds):
    ''' 

    '''

    pass


def main_consts():
    ''' 

    '''

    pass


def mergeUvIslands(islandList):
    ''' 

    '''

    pass


def optiRotateUvIsland(faces):
    ''' 

    '''

    pass


def packIslands(islandList):
    ''' 

    '''

    pass


def pointInIsland(pt, island):
    ''' 

    '''

    pass


def pointInTri2D(v, v1, v2, v3):
    ''' 

    '''

    pass


def rotate_uvs(uv_points, angle):
    ''' 

    '''

    pass



import sys
import typing
import gpu.types


def batch_for_shader(shader: 'gpu.types.GPUShader',
                     type: str,
                     content: dict,
                     indices=None):
    ''' Return a batch already configured and compatible with the shader.

    :param shader: shader for which a compatible format will be computed.
    :type shader: 'gpu.types.GPUShader'
    :param type: "'POINTS', 'LINES', 'TRIS' or 'LINES_ADJ'".
    :type type: str
    :param content: Maps the name of the shader attribute with the data to fill the vertex buffer.
    :type content: dict
    :return: compatible batch
    '''

    pass



import sys
import typing
import bpy.types


def add_and_link_node(
        type: str = "",
        use_transform: bool = False,
        settings: typing.
        Union[typing.Dict[str, 'bpy.types.NodeSetting'], typing.
              List['bpy.types.NodeSetting'], 'bpy_prop_collection'] = None,
        link_socket_index: int = 0):
    ''' Add a node to the active tree and link to an existing socket

    :param type: Node Type, Node type
    :type type: str
    :param use_transform: Use Transform, Start transform operator after inserting the node
    :type use_transform: bool
    :param settings: Settings, Settings to be applied on the newly created node
    :type settings: typing.Union[typing.Dict[str, 'bpy.types.NodeSetting'], typing.List['bpy.types.NodeSetting'], 'bpy_prop_collection']
    :param link_socket_index: Link Socket Index, Index of the socket to link
    :type link_socket_index: int
    '''

    pass


def add_file(filepath: str = "",
             filter_blender: bool = False,
             filter_backup: bool = False,
             filter_image: bool = True,
             filter_movie: bool = True,
             filter_python: bool = False,
             filter_font: bool = False,
             filter_sound: bool = False,
             filter_text: bool = False,
             filter_btx: bool = False,
             filter_collada: bool = False,
             filter_alembic: bool = False,
             filter_folder: bool = True,
             filter_blenlib: bool = False,
             filemode: int = 9,
             relative_path: bool = True,
             show_multiview: bool = False,
             use_multiview: bool = False,
             display_type: typing.Union[str, int] = 'DEFAULT',
             sort_method: typing.Union[str, int] = 'FILE_SORT_ALPHA',
             name: str = "Image"):
    ''' Add a file node to the current node editor

    :param filepath: File Path, Path to file
    :type filepath: str
    :param filter_blender: Filter .blend files
    :type filter_blender: bool
    :param filter_backup: Filter .blend files
    :type filter_backup: bool
    :param filter_image: Filter image files
    :type filter_image: bool
    :param filter_movie: Filter movie files
    :type filter_movie: bool
    :param filter_python: Filter python files
    :type filter_python: bool
    :param filter_font: Filter font files
    :type filter_font: bool
    :param filter_sound: Filter sound files
    :type filter_sound: bool
    :param filter_text: Filter text files
    :type filter_text: bool
    :param filter_btx: Filter btx files
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files
    :type filter_alembic: bool
    :param filter_folder: Filter folders
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file
    :type relative_path: bool
    :param show_multiview: Enable Multi-View
    :type show_multiview: bool
    :param use_multiview: Use Multi-View
    :type use_multiview: bool
    :param display_type: Display Type * DEFAULT Default, Automatically determine display type for files. * LIST_SHORT Short List, Display files as short list. * LIST_LONG Long List, Display files as a detailed list. * THUMBNAIL Thumbnails, Display files as thumbnails.
    :type display_type: typing.Union[str, int]
    :param sort_method: File sorting mode * FILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically. * FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type. * FILE_SORT_TIME Sort by time, Sort files by modification time. * FILE_SORT_SIZE Sort by size, Sort files by size.
    :type sort_method: typing.Union[str, int]
    :param name: Name, Data-block name to assign
    :type name: str
    '''

    pass


def add_mask(name: str = "Mask"):
    ''' Add a mask node to the current node editor

    :param name: Name, Data-block name to assign
    :type name: str
    '''

    pass


def add_node(
        type: str = "",
        use_transform: bool = False,
        settings: typing.
        Union[typing.Dict[str, 'bpy.types.NodeSetting'], typing.
              List['bpy.types.NodeSetting'], 'bpy_prop_collection'] = None):
    ''' Add a node to the active tree

    :param type: Node Type, Node type
    :type type: str
    :param use_transform: Use Transform, Start transform operator after inserting the node
    :type use_transform: bool
    :param settings: Settings, Settings to be applied on the newly created node
    :type settings: typing.Union[typing.Dict[str, 'bpy.types.NodeSetting'], typing.List['bpy.types.NodeSetting'], 'bpy_prop_collection']
    '''

    pass


def add_reroute(path: typing.Union[
        typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.
        List['bpy.types.OperatorMousePath'], 'bpy_prop_collection'] = None,
                cursor: int = 6):
    ''' Add a reroute node

    :param path: Path
    :type path: typing.Union[typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.List['bpy.types.OperatorMousePath'], 'bpy_prop_collection']
    :param cursor: Cursor
    :type cursor: int
    '''

    pass


def add_search(
        type: str = "",
        use_transform: bool = False,
        settings: typing.
        Union[typing.Dict[str, 'bpy.types.NodeSetting'], typing.
              List['bpy.types.NodeSetting'], 'bpy_prop_collection'] = None,
        node_item: typing.Union[str, int] = ''):
    ''' Add a node to the active tree

    :param type: Node Type, Node type
    :type type: str
    :param use_transform: Use Transform, Start transform operator after inserting the node
    :type use_transform: bool
    :param settings: Settings, Settings to be applied on the newly created node
    :type settings: typing.Union[typing.Dict[str, 'bpy.types.NodeSetting'], typing.List['bpy.types.NodeSetting'], 'bpy_prop_collection']
    :param node_item: Node Type, Node type
    :type node_item: typing.Union[str, int]
    '''

    pass


def attach():
    ''' Attach active node to a frame

    '''

    pass


def backimage_fit():
    ''' Fit the background image to the view

    '''

    pass


def backimage_move():
    ''' Move Node backdrop

    '''

    pass


def backimage_sample():
    ''' Use mouse to sample background image

    '''

    pass


def backimage_zoom(factor: float = 1.2):
    ''' Zoom in/out the background image

    :param factor: Factor
    :type factor: float
    '''

    pass


def clear_viewer_border():
    ''' Clear the boundaries for viewer operations

    '''

    pass


def clipboard_copy():
    ''' Copies selected nodes to the clipboard

    '''

    pass


def clipboard_paste():
    ''' Pastes nodes from the clipboard to the active node tree

    '''

    pass


def collapse_hide_unused_toggle():
    ''' Toggle collapsed nodes and hide unused sockets :file: startup/bl_operators/node.py\:260 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/node.py$260> _

    '''

    pass


def cryptomatte_layer_add():
    ''' Add a new input layer to a Cryptomatte node

    '''

    pass


def cryptomatte_layer_remove():
    ''' Remove layer from a Cryptomatte node

    '''

    pass


def delete():
    ''' Delete selected nodes

    '''

    pass


def delete_reconnect():
    ''' Delete nodes; will reconnect nodes as if deletion was muted

    '''

    pass


def detach():
    ''' Detach selected nodes from parents

    '''

    pass


def detach_translate_attach(NODE_OT_detach=None,
                            TRANSFORM_OT_translate=None,
                            NODE_OT_attach=None):
    ''' Detach nodes, move and attach to frame

    :param NODE_OT_detach: Detach Nodes, Detach selected nodes from parents
    :param TRANSFORM_OT_translate: Move, Move selected items
    :param NODE_OT_attach: Attach Nodes, Attach active node to a frame
    '''

    pass


def duplicate(keep_inputs: bool = False):
    ''' Duplicate selected nodes

    :param keep_inputs: Keep Inputs, Keep the input links to duplicated nodes
    :type keep_inputs: bool
    '''

    pass


def duplicate_move(NODE_OT_duplicate=None, NODE_OT_translate_attach=None):
    ''' Duplicate selected nodes and move them

    :param NODE_OT_duplicate: Duplicate Nodes, Duplicate selected nodes
    :param NODE_OT_translate_attach: Move and Attach, Move nodes and attach to frame
    '''

    pass


def duplicate_move_keep_inputs(NODE_OT_duplicate=None,
                               NODE_OT_translate_attach=None):
    ''' Duplicate selected nodes keeping input links and move them

    :param NODE_OT_duplicate: Duplicate Nodes, Duplicate selected nodes
    :param NODE_OT_translate_attach: Move and Attach, Move nodes and attach to frame
    '''

    pass


def find_node(prev: bool = False):
    ''' Search for named node and allow to select and activate it

    :param prev: Previous
    :type prev: bool
    '''

    pass


def group_edit(exit: bool = False):
    ''' Edit node group

    :param exit: Exit
    :type exit: bool
    '''

    pass


def group_insert():
    ''' Insert selected nodes into a node group

    '''

    pass


def group_make():
    ''' Make group from selected nodes

    '''

    pass


def group_separate(type: typing.Union[str, int] = 'COPY'):
    ''' Separate selected nodes from the node group

    :param type: Type * COPY Copy, Copy to parent node tree, keep group intact. * MOVE Move, Move to parent node tree, remove from group.
    :type type: typing.Union[str, int]
    '''

    pass


def group_ungroup():
    ''' Ungroup selected nodes

    '''

    pass


def hide_socket_toggle():
    ''' Toggle unused node socket display

    '''

    pass


def hide_toggle():
    ''' Toggle hiding of selected nodes

    '''

    pass


def insert_offset():
    ''' Automatically offset nodes on insertion

    '''

    pass


def join():
    ''' Attach selected nodes to a new common frame

    '''

    pass


def link(detach: bool = False):
    ''' Use the mouse to create a link between two nodes

    :param detach: Detach, Detach and redirect existing links
    :type detach: bool
    '''

    pass


def link_make(replace: bool = False):
    ''' Makes a link between selected output in input sockets

    :param replace: Replace, Replace socket connections with the new links
    :type replace: bool
    '''

    pass


def link_viewer():
    ''' Link to viewer node

    '''

    pass


def links_cut(path: typing.Union[
        typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.
        List['bpy.types.OperatorMousePath'], 'bpy_prop_collection'] = None,
              cursor: int = 9):
    ''' Use the mouse to cut (remove) some links

    :param path: Path
    :type path: typing.Union[typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.List['bpy.types.OperatorMousePath'], 'bpy_prop_collection']
    :param cursor: Cursor
    :type cursor: int
    '''

    pass


def links_detach():
    ''' Remove all links to selected nodes, and try to connect neighbor nodes together

    '''

    pass


def move_detach_links(NODE_OT_links_detach=None,
                      TRANSFORM_OT_translate=None,
                      NODE_OT_insert_offset=None):
    ''' Move a node to detach links

    :param NODE_OT_links_detach: Detach Links, Remove all links to selected nodes, and try to connect neighbor nodes together
    :param TRANSFORM_OT_translate: Move, Move selected items
    :param NODE_OT_insert_offset: Insert Offset, Automatically offset nodes on insertion
    '''

    pass


def move_detach_links_release(NODE_OT_links_detach=None,
                              NODE_OT_translate_attach=None):
    ''' Move a node to detach links

    :param NODE_OT_links_detach: Detach Links, Remove all links to selected nodes, and try to connect neighbor nodes together
    :param NODE_OT_translate_attach: Move and Attach, Move nodes and attach to frame
    '''

    pass


def mute_toggle():
    ''' Toggle muting of the nodes

    '''

    pass


def new_node_tree(type: typing.Union[str, int] = '', name: str = "NodeTree"):
    ''' Create a new node tree

    :param type: Tree Type
    :type type: typing.Union[str, int]
    :param name: Name
    :type name: str
    '''

    pass


def node_color_preset_add(name: str = "",
                          remove_name: bool = False,
                          remove_active: bool = False):
    ''' Add or remove a Node Color Preset

    :param name: Name, Name of the preset, used to make the path name
    :type name: str
    :param remove_name: remove_name
    :type remove_name: bool
    :param remove_active: remove_active
    :type remove_active: bool
    '''

    pass


def node_copy_color():
    ''' Copy color to all selected nodes

    '''

    pass


def options_toggle():
    ''' Toggle option buttons display for selected nodes

    '''

    pass


def output_file_add_socket(file_path: str = "Image"):
    ''' Add a new input to a file output node

    :param file_path: File Path, Sub-path of the output file
    :type file_path: str
    '''

    pass


def output_file_move_active_socket(direction: typing.Union[str, int] = 'DOWN'):
    ''' Move the active input of a file output node up or down the list

    :param direction: Direction
    :type direction: typing.Union[str, int]
    '''

    pass


def output_file_remove_active_socket():
    ''' Remove active input from a file output node

    '''

    pass


def parent_set():
    ''' Attach selected nodes

    '''

    pass


def preview_toggle():
    ''' Toggle preview display for selected nodes

    '''

    pass


def read_viewlayers():
    ''' Read all render layers of all used scenes

    '''

    pass


def render_changed():
    ''' Render current scene, when input node's layer has been changed

    '''

    pass


def resize():
    ''' Resize a node

    '''

    pass


def select(mouse_x: int = 0,
           mouse_y: int = 0,
           extend: bool = False,
           socket_select: bool = False,
           deselect_all: bool = False):
    ''' Select the node under the cursor

    :param mouse_x: Mouse X
    :type mouse_x: int
    :param mouse_y: Mouse Y
    :type mouse_y: int
    :param extend: Extend
    :type extend: bool
    :param socket_select: Socket Select
    :type socket_select: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor
    :type deselect_all: bool
    '''

    pass


def select_all(action: typing.Union[str, int] = 'TOGGLE'):
    ''' (De)select all nodes

    :param action: Action, Selection action to execute * TOGGLE Toggle, Toggle selection for all elements. * SELECT Select, Select all elements. * DESELECT Deselect, Deselect all elements. * INVERT Invert, Invert selection of all elements.
    :type action: typing.Union[str, int]
    '''

    pass


def select_box(tweak: bool = False,
               xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[str, int] = 'SET'):
    ''' Use box selection to select nodes

    :param tweak: Tweak, Only activate when mouse is not over a node - useful for tweak gesture
    :type tweak: bool
    :param xmin: X Min
    :type xmin: int
    :param xmax: X Max
    :type xmax: int
    :param ymin: Y Min
    :type ymin: int
    :param ymax: Y Max
    :type ymax: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param mode: Mode * SET Set, Set a new selection. * ADD Extend, Extend existing selection. * SUB Subtract, Subtract existing selection.
    :type mode: typing.Union[str, int]
    '''

    pass


def select_circle(x: int = 0,
                  y: int = 0,
                  radius: int = 25,
                  wait_for_input: bool = True,
                  mode: typing.Union[str, int] = 'SET'):
    ''' Use circle selection to select nodes

    :param x: X
    :type x: int
    :param y: Y
    :type y: int
    :param radius: Radius
    :type radius: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param mode: Mode * SET Set, Set a new selection. * ADD Extend, Extend existing selection. * SUB Subtract, Subtract existing selection.
    :type mode: typing.Union[str, int]
    '''

    pass


def select_grouped(extend: bool = False,
                   type: typing.Union[str, int] = 'TYPE'):
    ''' Select nodes with similar properties

    :param extend: Extend, Extend selection instead of deselecting everything first
    :type extend: bool
    :param type: Type
    :type type: typing.Union[str, int]
    '''

    pass


def select_lasso(
        tweak: bool = False,
        path: typing.Union[
            typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.
            List['bpy.types.OperatorMousePath'], 'bpy_prop_collection'] = None,
        mode: typing.Union[str, int] = 'SET'):
    ''' Select nodes using lasso selection

    :param tweak: Tweak, Only activate when mouse is not over a node - useful for tweak gesture
    :type tweak: bool
    :param path: Path
    :type path: typing.Union[typing.Dict[str, 'bpy.types.OperatorMousePath'], typing.List['bpy.types.OperatorMousePath'], 'bpy_prop_collection']
    :param mode: Mode * SET Set, Set a new selection. * ADD Extend, Extend existing selection. * SUB Subtract, Subtract existing selection.
    :type mode: typing.Union[str, int]
    '''

    pass


def select_link_viewer(NODE_OT_select=None, NODE_OT_link_viewer=None):
    ''' Select node and link it to a viewer node

    :param NODE_OT_select: Select, Select the node under the cursor
    :param NODE_OT_link_viewer: Link to Viewer Node, Link to viewer node
    '''

    pass


def select_linked_from():
    ''' Select nodes linked from the selected ones

    '''

    pass


def select_linked_to():
    ''' Select nodes linked to the selected ones

    '''

    pass


def select_same_type_step(prev: bool = False):
    ''' Activate and view same node type, step by step

    :param prev: Previous
    :type prev: bool
    '''

    pass


def shader_script_update():
    ''' Update shader script node with new sockets and options from the script

    '''

    pass


def switch_view_update():
    ''' Update views of selected node

    '''

    pass


def translate_attach(TRANSFORM_OT_translate=None,
                     NODE_OT_attach=None,
                     NODE_OT_insert_offset=None):
    ''' Move nodes and attach to frame

    :param TRANSFORM_OT_translate: Move, Move selected items
    :param NODE_OT_attach: Attach Nodes, Attach active node to a frame
    :param NODE_OT_insert_offset: Insert Offset, Automatically offset nodes on insertion
    '''

    pass


def translate_attach_remove_on_cancel(TRANSFORM_OT_translate=None,
                                      NODE_OT_attach=None,
                                      NODE_OT_insert_offset=None):
    ''' Move nodes and attach to frame

    :param TRANSFORM_OT_translate: Move, Move selected items
    :param NODE_OT_attach: Attach Nodes, Attach active node to a frame
    :param NODE_OT_insert_offset: Insert Offset, Automatically offset nodes on insertion
    '''

    pass


def tree_path_parent():
    ''' Go to parent node tree :file: startup/bl_operators/node.py\:290 <https://developer.blender.org/diffusion/B/browse/master/release/scripts/startup/bl_operators/node.py$290> _

    '''

    pass


def tree_socket_add(in_out: typing.Union[str, int] = 'IN'):
    ''' Add an input or output socket to the current node tree

    :param in_out: Socket Type
    :type in_out: typing.Union[str, int]
    '''

    pass


def tree_socket_move(direction: typing.Union[str, int] = 'UP'):
    ''' Move a socket up or down in the current node tree's sockets stack

    :param direction: Direction
    :type direction: typing.Union[str, int]
    '''

    pass


def tree_socket_remove():
    ''' Remove an input or output socket to the current node tree

    '''

    pass


def view_all():
    ''' Resize view so you can see all nodes

    '''

    pass


def view_selected():
    ''' Resize view so you can see selected nodes

    '''

    pass


def viewer_border(xmin: int = 0,
                  xmax: int = 0,
                  ymin: int = 0,
                  ymax: int = 0,
                  wait_for_input: bool = True,
                  deselect: bool = False,
                  extend: bool = True):
    ''' Set the boundaries for viewer operations

    :param xmin: X Min
    :type xmin: int
    :param xmax: X Max
    :type xmax: int
    :param ymin: Y Min
    :type ymin: int
    :param ymax: Y Max
    :type ymax: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param deselect: Deselect, Deselect rather than select items
    :type deselect: bool
    :param extend: Extend, Extend selection instead of deselecting everything first
    :type extend: bool
    '''

    pass
